package ru.dbotthepony.mc.otm.client.screen.tech

import net.minecraft.network.chat.Component
import net.minecraft.world.entity.player.Inventory
import ru.dbotthepony.mc.otm.client.screen.MatteryScreen
import ru.dbotthepony.mc.otm.core.TranslatableComponent
import ru.dbotthepony.mc.otm.client.screen.panels.*
import ru.dbotthepony.mc.otm.client.screen.panels.button.ButtonPanel
import ru.dbotthepony.mc.otm.client.screen.panels.input.NetworkNumberInputPanel
import ru.dbotthepony.mc.otm.core.util.formatPower
import ru.dbotthepony.mc.otm.menu.tech.EnergyCounterMenu

class EnergyCounterScreen(menu: EnergyCounterMenu, inventory: Inventory, title: Component) : MatteryScreen<EnergyCounterMenu>(menu, inventory, title) {
	override fun makeMainFrame(): FramePanel<out MatteryScreen<*>> {
		val frame = super.makeMainFrame()!!

		var label: Label<EnergyCounterScreen> = object : Label<EnergyCounterScreen>(this@EnergyCounterScreen, frame) {
			override fun tick() {
				super.tick()
				text = TranslatableComponent(
					"otm.item.power.passed",
					menu.passed.formatPower()
				)
			}
		}

		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		label = object : Label<EnergyCounterScreen>(this@EnergyCounterScreen, frame) {
			override fun tick() {
				super.tick()
				text = TranslatableComponent(
					"otm.item.power.average",
					menu.average.formatPower()
				)
			}
		}

		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		label = object : Label<EnergyCounterScreen>(this@EnergyCounterScreen, frame) {
			override fun tick() {
				super.tick()
				text = TranslatableComponent(
					"otm.item.power.last_20_ticks",
					menu.last20Ticks.formatPower()
				)
			}
		}

		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		label = object : Label<EnergyCounterScreen>(this@EnergyCounterScreen, frame) {
			override fun tick() {
				super.tick()
				text = TranslatableComponent(
					"otm.item.power.last_tick",
					menu.lastTick.formatPower()
				)
			}
		}

		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		label = object : Label<EnergyCounterScreen>(this@EnergyCounterScreen, frame) {
			override fun tick() {
				super.tick()

				text = TranslatableComponent(
					"block.overdrive_that_matters.energy_counter.facing",
					menu.inputDirection
				)
			}
		}

		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		if (!menu.ply.isSpectator) {
			val button = ButtonPanel(this, frame, 0f, 0f, 0f, 20f, TranslatableComponent("block.overdrive_that_matters.energy_counter.switch"))
			button.dock = Dock.TOP
			button.setDockMargin(4f, 0f, 4f, 0f)
			button.bind { menu.switchDirection.input(null) }
		}

		val infoPanels = frame.fetchChildren()

		label = Label(this, frame, TranslatableComponent("block.overdrive_that_matters.energy_counter.limit"))
		label.dock = Dock.TOP
		label.setDockMargin(4f, 0f, 0f, 0f)

		val inputPanel = NetworkNumberInputPanel(this, frame, widget = menu.maxIOInput, networkValue = menu::maxIO)
		inputPanel.dock = Dock.TOP
		inputPanel.setDockMargin(4f, 0f, 4f, 0f)

		val limitsPanels = frame.fetchChildren().filter { !infoPanels.contains(it) }

		val informationTab = frame.Tab()
		val limitsTab = frame.Tab()

		informationTab.showHidePanels(infoPanels)
		limitsTab.showHidePanels(limitsPanels)

		limitsTab.onClose!!.run()

		return frame
	}
}
