package ru.dbotthepony.mc.otm.menu.input

import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import kotlin.reflect.KMutableProperty0

abstract class AbstractNetworkedInput<V> {
	abstract val input: MatteryMenu.PlayerInput<V>
	abstract val value: V

	var supplier: (() -> V)? = null
	var consumer: ((V) -> Unit)? = null

	fun withSupplier(func: () -> V): AbstractNetworkedInput<V> {
		supplier = func
		return this
	}

	fun withConsumer(func: (V) -> Unit): AbstractNetworkedInput<V> {
		consumer = func
		return this
	}

	fun with(state: KMutableProperty0<V>): AbstractNetworkedInput<V> {
		withConsumer { state.set(it) }
		withSupplier { state.get() }
		return this
	}

	fun clear(): AbstractNetworkedInput<V> {
		supplier = null
		consumer = null
		return this
	}

	/**
	 * shortcut to checked input of [input]
	 */
	fun input(newValue: V) {
		input.checkedInput(newValue, minecraft.player)
	}

	/**
	 * checks if local (client) player can modify this input
	 */
	fun check(): Boolean {
		return input.allowSpectators || minecraft.player?.isSpectator == false
	}
}
